<?php

namespace App\Http\Controllers;

use App\Models\Usecase;
use App\Models\Document;
use App\Models\PointHistory;
use Illuminate\Http\Request;
use App\Models\DocumentRequest;
use App\Components\Drivers\OpenAi;
use App\Helpers\Classes\ArtisanApi;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;

class DocumentController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request, $document_id = null,)
    {
        $usecases = Usecase::active()->textCase()
            ->withTranslation()
            ->when(Auth::user()->hasActiveSubscription(), function ($query) {
                $authorized = Auth::user()->getActiveSubscription()->plan->usecases->pluck('id');
                $query->whereIn('id', $authorized);
            })
            ->get();

        $langs = openai_languages();
        $tones = get_tones();
        $variants = get_variants();
        $creativities = get_writing_styles();
        $search = $request->get('q', null);

        $documents = Document::query()
            ->when(!empty($search), function ($query) use ($search) {
                $query->search($search,);
            })
            ->where('user_id', Auth::user()->id)
            ->textCase()
            ->latest()
            ->paginate(10);

        $selected_document = null;
        if ($document_id != null) {
            $selected_document = Document::query()
                ->with(['requests' => function ($query) {
                    $query->latest();
                }])
                ->where('id', $document_id)
                ->first();

            if ($selected_document->user_id != Auth::user()->id) {
                return redirect()->back();
            }
        }

        $selected_usecase = null;
        if (isset($request->usecase)) {
            $selected_usecase = Usecase::find($request->usecase);
        }

        return view('document.index', compact('selected_usecase', 'usecases', 'langs', 'tones', 'variants', 'creativities', 'documents', 'selected_document'));
    }

    public function getFields(Request $request)
    {
        $usecase = Usecase::findOrFail($request->id);
        $view = view('document.fields', compact('usecase'))->render();

        return response()->json(['view' => $view]);
    }

    public function storeDocument(Request $request, Document $document = null)
    {
        $user  = Auth::user();
        if (empty(config('artisan.openai_api_key'))) {
            return redirect()->back()->withError(__('admin.emptyAiKey'));
        }

        if (!app(ArtisanApi::class)->hasRegistered()) {
            return redirect()->back()->withError("\120\x6c\x65\141\163\x65\40\162\145\x67\x69\x73\x74\145\162\40\171\157\165\x72\40\160\x75\162\143\150\141\163\x65\x2e");
        }

        Session::put('document_request', $request->except('_token', 'document_type', 'usecase_id'));
        if ($request->input('document_type', null) === 'image') {
            $usecase = Usecase::withTranslation()->withCount('usageToday')->imageCase()->firstOrFail();
        } else {
            $usecase = Usecase::withTranslation()->withCount('usageToday')->findOrFail($request->usecase_id);
        }

        $validate = $this->checkPropertyValidity($usecase);
        if (!$validate['status']) {
            if ($request->wantsJson()) {
                return response()->json(['success' => false, 'data' => null, 'message' => $validate['message']]);
            }

            return redirect()->back()->withError($validate['message']);
        }

        $result = new OpenAi();
        /// image model processing
        if ($usecase->usecase_type == "image") {
            $prompt = $this->getImagePrompt($request->usecase_id, $request);
            $number = $request->no_of_image;
            $resolution = $request->resolution;
            $response = $result->parseImage($prompt['prompt'], $number, $resolution);
            if (!$response['success'] || $prompt == null) {
                $errMessage = $response['success'] ?? '';
                if ($request->wantsJson()) {
                    return response()->json(['success' => false, 'data' => null, 'message' => $errMessage]);
                }

                return redirect()->route('aiimages.index')->withErrors($errMessage);
            }
        }

        /// text model document processing
        else {
            $promptResponse = $this->getPrompt($request->usecase_id, $request);
            if ($promptResponse['status'] == false) {
                if ($request->wantsJson()) {
                    return response()->json(['success' => false, 'data' => null, 'message' => $promptResponse['message']]);
                }
                return redirect()->back()->withErrors($promptResponse['message']);
            }

            $prompt = $promptResponse['prompt'];

            $response = $result->parse($prompt);
            if (!$response['success'] || $prompt == null) {
                $errMessage = $response['success'] ?? '';
                if ($request->wantsJson()) {
                    return response()->json(['success' => false, 'data' => null, 'message' => $errMessage]);
                }

                return redirect()->back()->withErrors($errMessage);
            }
        }

        $usecase->createVisitLog(auth()->user());

        if (!$document) {
            $documentArray = [
                'user_id' => auth()->id(),
                'name' => "Untitle Document",
            ];

            $document = Document::create($documentArray);
        }
        $requestArray = [
            'document_id' => $document->id,
            'result' => $response['text'] ?? "",
            'tokens' => $response['usage']['total_tokens'] ?? 0,
            'no_of_words' => str_word_count($response['text'] ?? ''),
            'no_of_images' => ($usecase->usecase_type == "image") ? $response['usage'] : 0,
        ];

        $document->no_of_words = $document->no_of_words + str_word_count($response['text'] ?? '');
        $document->tokens = $document->tokens + ($response['usage']['total_tokens'] ?? 0);
        $document->no_of_images = $usecase->usecase_type == "image" ? $document->no_of_images + $response['usage'] : $document->no_of_images ?? 0;
        $document->document_type = ($usecase->usecase_type == "image") ? "image" : "text";
        $document->name = ($request->name != null) ? $request->name : $document->name;
        $document->update();
        $documentRequest = DocumentRequest::create($requestArray);

        $point_history = new PointHistory();
        $point_history->no_of_words = $documentRequest->no_of_words;
        $point_history->no_of_images = $documentRequest->no_of_images;
        $point_history->type = 1;
        $point_history->user_id = $user->id;
        $point_history->save();

        $user->no_of_words  = ($user->no_of_words - $documentRequest->no_of_words) > 0 ? $user->no_of_words - $documentRequest->no_of_words : 0;
        $user->no_of_images  = ($user->no_of_images -  $documentRequest->no_of_images) > 0 ? $user->no_of_images -  $documentRequest->no_of_images : 0;
        $user->save();

        $sessionRequest = Session::get('document_request', []);

        if ($usecase->usecase_type == "image") {
            $html = "";
            if ($response['success'] == true && count($response['images']) > 0) {
                foreach ($response['images'] as $image) {
                    $url = $image['url'];
                    $img = $documentRequest->addMediaFromUrl($url)->toMediaCollection('document-requests');
                    $html .= "![image]({$img->getUrl()}) ";
                }
            }
            $documentRequest->result = $html;
            $documentRequest->update();

            return redirect()->route('aiimages.index');
        }

        if ($request->wantsJson()) {
            return response()->json(['success' => true, 'data' => $response['text'] ?? "", 'message' => "successfull", 'no_of_images' => Auth::user()->remainingImages(), 'no_of_words' => Auth::user()->remainingWords()]);
        }

        return redirect()->route('document.index', ['document_id' => $document->id]);
    }

    private function getPrompt($usecase_id, $request)
    {
        $return_arry = [];
        $usecase = Usecase::where('id', $usecase_id)->first();
        $prompt = $usecase->command;
        if (!empty($request->tone)) {
            $prompt .= ". Your reply must be in [tone] tone";
        }

        if (!empty($request->style)) {
            $prompt .= ". Your writting style should be [style]";
        }

        if ($usecase->fields != null) {
            foreach ($usecase->fields as $field) {
                if ($field->short_code != null) {
                    $value = $field->id;
                    $required = $field->required;
                    if ($required == 1 && $request->values[$value] == null) {
                        $return_arry = ['message' => __('document.requiredError', ['name' => $field->label]), 'status' => false];

                        return $return_arry;
                    }
                    $prompt = str_replace($field->short_code, html_entity_decode($request->values[$value], ENT_QUOTES), $prompt);
                }
            }
        }

        $prompt = str_replace("[language]", $request->language, $prompt);
        $prompt = str_replace("[tone]", $request->tone, $prompt);
        $prompt = str_replace("[variant]", $request->variant, $prompt);
        $prompt = str_replace("[style]", $request->style, $prompt);

        $return_arry = ['status' => true, 'prompt' => $prompt];

        return $return_arry;
    }

    private function getImagePrompt($usecase_id, $request)
    {
        $return_arry = [];
        $usecase = Usecase::where('id', $usecase_id)->first();
        $prompt = "Create a digital art with following description: " . $request->description;;
        if ($request->art_style != "None") {
            $prompt .= " n\nSet art style as: {$request->art_style}";
        }

        if ($request->medium != "None") {
            $prompt .= "\n\nSet art medium as: {$request->medium}";
        }

        if ($request->mood != "None") {
            $prompt .= "\n\nSet art mood as: {$request->mood}";
        }

        $prompt .= "\n\n";

        $return_arry = ['status' => true, 'prompt' => $prompt];

        return $return_arry;
    }

    public function checkPropertyValidity($usecase)
    {
        $user = Auth::user();
        $checkUserSubscription = $user->hasActiveSubscription();
        $userPlan = $user->getActiveSubscription()->plan ?? null;
        $usageCountLimit = $checkUserSubscription ? $userPlan->usecase_daily_limit : setting('usecase_daily_limit', '100');
        $wordCountLimit = $user->remainingWords();
        $imageCountLimit = $user->remainingImages();
        if ($usageCountLimit  <= $usecase->usage_today_count) {
            return ['status' => false, 'message' => __('document.limitExceed')];
        }

        if ($usecase->usecase_type == "image") {
            if ($imageCountLimit < 1) {
                return ['status' => false, 'message' => __('document.limitExceedImages')];
            }
        } else {
            if ($wordCountLimit < 1) {
                return ['status' => false, 'message' => __('document.limitExceedWords')];
            }
        }

        return ['status' => true, 'message' => ''];
    }


    public function checkPropertyValidityOld($usecase)
    {
        $checkUserSubscription = Auth::user()->hasActiveSubscription();
        $userPlan = Auth::user()->getActiveSubscription()->plan ?? null;
        $usageCountLimit = $checkUserSubscription ? $userPlan->usecase_daily_limit : setting('usecase_daily_limit', '100');
        $wordCountLimit = $checkUserSubscription ? $userPlan->no_of_words : setting('no_of_words', '100');
        $imageCountLimit = $checkUserSubscription ? $userPlan->no_of_words : setting('no_of_images', '100');

        if ($usageCountLimit  <= $usecase->usage_today_count) {
            return ['status' => false, 'message' => __('document.limitExceed')];
        }

        if ($usecase->usecase_type == "image") {
            if (Auth::user()->documentImages() >= $imageCountLimit) {
                return ['status' => false, 'message' => __('document.limitExceedImages')];
            }
        } else {
            if (Auth::user()->documentWords() >= $wordCountLimit) {
                return ['status' => false, 'message' => __('document.limitExceedWords')];
            }
        }

        return ['status' => true, 'message' => ''];
    }

    public function checkUsage($usecase)
    {
        $usageCountLimit = Auth::user()->hasActiveSubscription() ? Auth::user()->getActiveSubscription()->plan->usecase_daily_limit : setting('usecase_daily_limit', '100');

        if ($usageCountLimit  <= $usecase->usage_today_count) {
            return false;
        }

        return true;
    }

    public function favouriteAction(Document $document)
    {
        $document->is_favourite = ($document->is_favourite == 1) ? 0 : 1;
        $document->update();
        return redirect()->back()->withSucccess(__('document.favouriteActionDone'));
    }

    public function updateDocument(Request $request, Document $document = null)
    {
        $document->name = $request->name;
        $document->update();

        if ($request->wantsJson()) {
            return response()->json(['success' => true, 'message' => __('document.documentUpdated')]);
        }

        return redirect()->back()->withSucccess(__('document.documentUpdated'));
    }

    public function downloadDocument(Document $document)
    {
        $content = '';

        foreach ($document->requests as $request) {
            $content .= wpautop($request->result) . "<br>&mdash;<br>";
        }

        $headers = array(
            "Content-type" => "text/html",
            "Content-Disposition" => "attachment;Filename={$document->name}.doc"
        );

        return response('<html><body>' . $content . '</body></html>', 200, $headers);
    }

    public function deleteDocument(Document $document = null)
    {
        $document->delete();

        return redirect()->back()->withSucccess(__('document.documentDeleted'));
    }
}
